@@ -656,15 +656,14 @@ the actual number of articles toggled is returned."
 		  (gnus-agent-group-path group) "/"))
 	    (date (gnus-time-to-day (current-time)))
 	    (case-fold-search t)
-	    pos alists crosses id elem)
+	    pos crosses id elem)
 	(gnus-make-directory dir)
 	(gnus-message 7 "Fetching articles for %s..." group)
 	;; Fetch the articles from the backend.
 	(if (gnus-check-backend-function 'retrieve-articles group)
 	    (setq pos (gnus-retrieve-articles articles group))
 	  (nnheader-temp-write nil
-	    (let ((buf (current-buffer))
-		  article)
+	    (let (article)
 	      (while (setq article (pop articles))
 		(when (gnus-request-article article group)
 		  (goto-char (point-max))
@@ -751,39 +750,33 @@ the actual number of articles toggled is returned."
 	(insert "\n"))
       (pop gnus-agent-group-alist))))
 
-(defun gnus-agent-fetch-headers (group articles &optional force)
-  (gnus-agent-load-alist group)
-  ;; Find out what headers we need to retrieve.
-  (when articles
-    (while (and articles
-		(assq (car articles) gnus-agent-article-alist))
-      (pop articles))
-    (let ((arts articles))
-      (while (cdr arts)
-	(if (assq (cadr arts) gnus-agent-article-alist)
-	    (setcdr arts (cddr arts))
-	  (setq arts (cdr arts)))))
-    ;; Fetch them.
-    (when articles
-      (gnus-message 7 "Fetching headers for %s..." group)
-      (save-excursion
-	(set-buffer nntp-server-buffer)
-	(unless (eq 'nov (gnus-retrieve-headers articles group))
-	  (nnvirtual-convert-headers))
-	;; Save these headers for later processing.
-	(copy-to-buffer gnus-agent-overview-buffer (point-min) (point-max))
-	(let (file)
-	  (when (file-exists-p
-		 (setq file (gnus-agent-article-name ".overview" group)))
-	    (gnus-agent-braid-nov group articles file))
-	  (gnus-make-directory (nnheader-translate-file-chars
-				(file-name-directory file)))
-	  (write-region (point-min) (point-max) file nil 'silent)
-	  (gnus-agent-save-alist group articles nil)
-	  (gnus-agent-enter-history "last-header-fetched-for-session"
-				    (list (cons group (nth (- (length  articles) 1) articles)))
-				    (gnus-time-to-day (current-time)))
-	  t)))))
+(defun gnus-agent-fetch-headers (group &optional force)
+  (when (gnus-agent-load-alist group)
+    (let ((articles (gnus-uncompress-range 
+		     (cons (1+ (caar (last (gnus-agent-load-alist group))))
+			   (cdr (gnus-active group))))))
+      ;; Fetch them.
+      (when articles
+	(gnus-message 7 "Fetching headers for %s..." group)
+	(save-excursion
+	  (set-buffer nntp-server-buffer)
+	  (unless (eq 'nov (gnus-retrieve-headers articles group))
+	    (nnvirtual-convert-headers))
+	  ;; Save these headers for later processing.
+	  (copy-to-buffer gnus-agent-overview-buffer (point-min) (point-max))
+	  (let (file)
+	    (when (file-exists-p
+		   (setq file (gnus-agent-article-name ".overview" group)))
+	      (gnus-agent-braid-nov group articles file))
+	    (gnus-make-directory (nnheader-translate-file-chars
+				  (file-name-directory file)))
+	    (write-region (point-min) (point-max) file nil 'silent)
+	    (gnus-agent-save-alist group articles nil)
+	    (gnus-agent-enter-history
+	     "last-header-fetched-for-session"
+	     (list (cons group (nth (- (length  articles) 1) articles)))
+	     (gnus-time-to-day (current-time)))
+	    articles))))))
 
 (defsubst gnus-agent-copy-nov-line (article)
   (let (b e)
@@ -843,7 +836,7 @@ the actual number of articles toggled is returned."
 	   (gnus-agent-article-name ".agentview" group)))))
 
 (defun gnus-agent-save-alist (group &optional articles state dir)
-  "Load the article-state alist for GROUP."
+  "Save the article-state alist for GROUP."
   (nnheader-temp-write (if dir
 			   (concat dir ".agentview")
 			 (gnus-agent-article-name ".agentview" group))
@@ -893,12 +886,11 @@ the actual number of articles toggled is returned."
   (let ((gnus-command-method method)
 	gnus-newsgroup-dependencies gnus-newsgroup-headers
 	gnus-newsgroup-scored gnus-headers gnus-score
-	gnus-use-cache articles score arts
+	gnus-use-cache articles arts
 	category predicate info marks score-param)
     ;; Fetch headers.
     (when (and (or (gnus-active group) (gnus-activate-group group))
-	       (setq articles (gnus-list-of-unread-articles group))
-	       (gnus-agent-fetch-headers group articles))
+	       (setq articles (gnus-agent-fetch-headers group)))
       ;; Parse them and see which articles we want to fetch.
       (setq gnus-newsgroup-dependencies
 	    (make-vector (length articles) 0))
@@ -966,8 +958,8 @@ the actual number of articles toggled is returned."
 (defvar gnus-category-buffer "*Agent Category*")
 
 (defvar gnus-category-line-format-alist
-  `((?c name ?s)
-    (?g groups ?d)))
+  `((?c gnus-tmp-name ?s)
+    (?g gnus-tmp-groups ?d)))
 
 (defvar gnus-category-mode-line-format-alist
   `((?u user-defined ?s)))
@@ -1043,15 +1035,15 @@ The following commands are available:
 (defalias 'gnus-category-position-point 'gnus-goto-colon)
 
 (defun gnus-category-insert-line (category)
-  (let* ((name (car category))
-	 (groups (length (cadddr category))))
+  (let* ((gnus-tmp-name (car category))
+	 (gnus-tmp-groups (length (cadddr category))))
     (beginning-of-line)
     (gnus-add-text-properties
      (point)
      (prog1 (1+ (point))
        ;; Insert the text.
        (eval gnus-category-line-format-spec))
-     (list 'gnus-category name))))
+     (list 'gnus-category gnus-tmp-name))))
 
 (defun gnus-enter-category-buffer ()
   "Go to the Category buffer."
