@@ -272,9 +272,12 @@ to virtual article number.")
 (deffoo nnvirtual-request-type (group &optional article)
   (if (not article)
       'unknown
-    (let ((mart (nnvirtual-map-article article)))
-      (when mart
-	(gnus-request-type (car mart) (cdr mart))))))
+    (if (numberp article)
+	(let ((mart (nnvirtual-map-article article)))
+	  (if mart
+	      (gnus-request-type (car mart) (cdr mart))))
+      (gnus-request-type
+       nnvirtual-last-accessed-component-group nil))))
 
 (deffoo nnvirtual-request-update-mark (group article mark)
   (let* ((nart (nnvirtual-map-article article))
@@ -399,7 +402,7 @@ to virtual article number.")
       (replace-match "" t t))
     (goto-char (point-min))
     (when (re-search-forward
-	   (concat (gnus-group-real-name group) ":[0-9]+")
+	   (concat (regexp-quote (gnus-group-real-name group)) ":[0-9]+")
 	   nil t)
       (replace-match "" t t))
     (unless (= (point) (point-max))
